# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines functions that may be useful when working with iterators.'''
# todo: make something like `filter` except it returns first found, or raises
# exception

import itertools
import builtins

from garlicsim.general_misc.infinity import Infinity


def consecutive_pairs(iterable):
    '''
    Iterate over successive pairs from the iterable.
    
    Example: if the iterable is [0, 1, 2, 3], then its `consecutive_pairs` would
    be [(0, 1), (1, 2), (2, 3)]. (Except it would be an iterator and not an
    actual list.)
    '''
    
    first_run = True
    old = None
    for current in iterable:
        if not first_run:
            yield (old, current)
        else:
            first_run = False
        old = current

        
def orderless_combinations(iterable, n, start=0):
    '''
    Iterate over combinations of items from the iterable.

    `n` specifies the number of items. `start` specifies the member number from
    which to start giving combinations. (Keep the default of 0 for doing the
    whole iterable.)
    
    Example:
    
    orderless_combinations([1, 2, 3, 4], n=2) would be, in list form: [[1, 2],
    [1, 3], [1, 4], [2, 3], [2, 4], [3, 4]].
    '''
    # todo: optimize or find 3rd party tool
    
    if n == 1:
        for thing in itertools.islice(iterable, start, None):
            yield [thing]
    for (i, thing) in itertools.islice(enumerate(iterable), start, None):
        for sub_result in orderless_combinations(iterable, n-1, start=i+1):
            yield [thing] + sub_result
    
    
def shorten(iterable, n):
    '''
    Shorten an iterator to length `n`.
    
    Iterate over the given iterable, but stop after `n` iterations (Or when the
    iterable stops iteration by itself.)
    
    `n` may be infinite.
    '''

    if n == Infinity:
        for thing in iterable:
            yield thing
        raise StopIteration
    
    assert isinstance(n, int)
    counter = 0
    for thing in iterable:
        if counter >= n: raise StopIteration
        yield thing
        counter += 1
        
        
def enumerate(reversable, reverse_index=False):
    '''
    Iterate over `(i, item)` pairs, where `i` is the index number of `item`.
    
    This is an extension of the builtin `enumerate`. What it allows is to get a
    reverse index, by specifying `reverse_index=True`. This causes `i` to count
    down to zero instead of up from zero, so the `i` of the last member will be
    zero.
    '''
    if reverse_index is False:
        return builtins.enumerate(reversable)
    else:
        my_list = list(builtins.enumerate(reversed(reversable)))
        my_list.reverse()
        return my_list

    
def is_iterable(thing):
    '''Return whether an object is iterable.'''
    return hasattr(thing, '__iter__')


def get_length(iterable):
    '''Get the length of an iterable.'''
    i = 0
    for thing in iterable:
        i += 1
    return i


def product(*args, **kwargs):
    '''
    Cartesian product of input iterables.

    Equivalent to nested for-loops in a generator expression. For example,
    `product(A, B)` returns the same as ((x,y) for x in A for y in B).
    
    product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
    product(range(2), repeat=3) --> 000 001 010 011 100 101 110 111
    '''
    # todo: revamp
    pools = map(tuple, args) * kwargs.get('repeat', 1)
    result = [[]]
    for pool in pools:
        result = [x + [y] for x in result for y in pool]
    for prod in result:
        yield tuple(prod)


def iter_with(iterable, context_manager):
    '''Iterate on `iterable`, `with`ing the context manager on every `next`.'''
    
    while True:
        
        with context_manager:
            next_item = next(iterable)
            # You may notice that we are not `except`ing a StopIteration here;
            # If we get one, it'll just get propagated and end *this* iterator.
        
        yield next_item
        
        


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the NodeRange class.

See its documentation for more info.
'''

from garlicsim.general_misc import cute_iter_tools
from garlicsim.general_misc import misc_tools

from .node import Node
from .block import Block

class NodeRange(object):
    '''A consecutive range of nodes.'''

    def __init__(self, start, end):
        '''
        Construct a NodeRange.
        
        `start` is the node or block in which this range starts.
        
        `end` is the node or block in which this range ends.
        '''
        
        self.start = start
        '''The node or block in which this range starts.'''
        
        self.end = end
        '''The node or block in which this range ends.'''
    
        
    def make_path(self):
        '''Make a path that goes through this node range.'''
        node_around_end = self.end if isinstance(self.end, Node) else \
                          self.end[0]
        return node_around_end.make_containing_path()

    
    def _sanity_check(self):
        '''
        Assert there are no obvious problems with this node range.
        
        This checks that the end node/block is a descendent of the start
        node/block.
        '''
        path = self.make_path()
        assert (self.start in path.__iter__(end=self.end))
        
    def __iter__(self):
        '''Iterate on the nodes in this range.'''
        return self.make_path().__iter__(start=self.start, end=self.end)
    
    def iterate_blockwise(self):
        '''
        Iterate on the nodes in this range, returning blocks where possible.
        '''
        path = self.make_path()
        return path.iterate_blockwise(start=self.start, end=self.end)

    def __contains__(self, node):
        path = self.make_path()
        return path.__contains__(node, start=self.start, end=self.end)
    
    def clone_with_blocks_dissolved(self):
        '''
        Make a node range that is specified with nodes and not blocks.
        
        A node range will be constructed that in this point of time is
        equivalent to the original node range, but whose `start` and `end` are
        specified as nodes and not as blocks.
        '''
        if isinstance(self.start, Block):
            new_start = self.start[0]
        else:
            new_start = self.start
        if isinstance(self.end, Block):
            new_end = self.end[-1]
        else:
            new_end = self.end
        
        return NodeRange(new_start, new_end)
    
    def get_outside_children(self):
        '''
        Get all the non-member children nodes of nodes which are members.
        
        This returns every node which is (a) a child of a node in this node
        range and (b) not in this node range itself.
        '''
        outside_children = []
        for thing in self.iterate_blockwise():
            candidate = thing if isinstance(thing, Node) else thing[-1]
            outside_children += [child for child in candidate.children if child
                                 not in self]
        return outside_children
            
    def copy(self):
        '''Shallow-copy the node range.'''
        klass = type(self)
        return klass(self.start, self.end)

    __copy__ = copy
    
    def __repr__(self):
        '''
        Get a string representation of the node range.
        
        Example output:        
        <garlicsim.data_structures.NodeRange, from node with clock 2
        to block that ends at clock 102, containing 101 nodes total, at
        0x291c550>
        '''
        return '<%s, from %s %s to %s %s, containing %s nodes total, at %s>' \
               % (
                   
                   misc_tools.shorten_class_address(
                       self.__class__.__module__,
                       self.__class__.__name__
                       ),
                   
                   'block that starts at clock' if isinstance(self.start, Block) \
                   else 'node with clock',
                   
                   self.start[0].state.clock if isinstance(self.start, Block) \
                   else self.start.state.clock,
                   
                   'block that ends at clock' if \
                   isinstance(self.end, Block) else 'node with clock',
                   
                   self.end[-1].state.clock if isinstance(self.end, Block) \
                   else self.end.state.clock,
                   
                cute_iter_tools.get_length(self),
                
                hex(id(self))
               )
    
    def __eq__(self, other):
        if not isinstance(other, NodeRange):
            return False
        r1 = self.clone_with_blocks_dissolved()
        r2 = other.clone_with_blocks_dissolved()
        return (r1.start is r2.start) and (r1.end is r2.end)
    
    def __req__(self, other):
        return self.__eq__(other)
    
    def __hash__(self):
        # Defining __hash__ because there's __eq__ which makes the default
        # __hash__ disappear on Python 3.
        return id(self)


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the ChangeTracker class.

See its documentation for more information.
'''

import weakref
import pickle


class ChangeTracker(object): 
    '''
    Tracks changes in objects that are registered with it.
    
    To register an object, use .check_in(obj). It will return True. Every time
    .check_in will be called with the same object, it will return whether the
    object changed since the last time it was checked in.
    '''
    
    def __init__(self):
        self.library = weakref.WeakKeyDictionary()
        
    def check_in(self, thing):
        '''        
        Checks in an object for change tracking.
        
        The first time you check in an object, it will return True. Every time
        .check_in will be called with the same object, it will return whether the
        object changed since the last time it was checked in.
        '''
        
        new_pickle = pickle.dumps(thing)
        
        if thing not in self.library:
            self.library[thing] = new_pickle
            return True
        
        # thing in self.library
        
        previous_pickle = self.library[thing]
        if previous_pickle == new_pickle:
            return False
        else:
            self.library[thing] = new_pickle
            return True
    
    def __contains__(self, thing):
        return self.library.__contains__(thing)




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This package defines a class ReadWriteLock in the module read_write_lock.
See that class's documentation.
'''
from .read_write_lock import *



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the CopyMode class.

See its documentation for more information.
'''


class CopyMode(dict):
    '''
    Passed as a memo to `deepcopy` to specify how objects should be copied.

    This type is meant to be subclassed. `__deepcopy__` methods may check which
    class the memo is to know what kind of deepcopying they should do.
    
    Typical usage:
    
    ##############################################
    
    class NetworkStyleCopying(CopyMode): pass
        
    class Something(object):
        def __deepcopy__(self, memo):
            if isinstance(memo, NetworkStlyeCopying):
                # Do network-style copying, whatever that means
            else:
                # Do normal copying
                
    s = Something()
    
    new_copy = copy.deepcopy(s, NetworkStyleCopying())
    # Now the new copy will be created using network style copying
    
    ##############################################
    '''
    __repr__ = object.__repr__


import os.path
import sys

try:
    import enthought
    import enthought.mayavi.tools.mlab_scene_model
finally:
    print((enthought.__file__))
    sys.stdout.flush()

import multiprocessing

def f():
    return 7

if __name__ == '__main__':
    p = multiprocessing.Process(target=f)
    p.start()
    p.join()



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the NodeSelection class.

See its documentation for more info.
'''

from garlicsim.misc import GarlicSimException
from garlicsim.general_misc import cute_iter_tools

from .node import Node
from .node_range import NodeRange

__all__ = ['NodeSelection']


class CompletelyCompact(GarlicSimException):
    '''The NodeSelection is already completely compact.'''
    

class NodeSelection(object):
    '''
    A selection of nodes.
    
    A NodeSelection could be described as a "set" of nodes, though the nodes are
    not specified one by one, but as a collection of node ranges.
    '''
    def __init__(self, ranges=()):
        '''
        Construct the NodeSelection.
        
        `ranges` is a list of node ranges that this selection will be made of.
        '''
        self.ranges = [ranges] if isinstance(ranges, NodeRange) else \
                      list(ranges)
        
    def compact(self):
        '''
        Compact the NodeSelection.
        
        This'll make it use the minimum number of node ranges while still
        containing exactly the same nodes.
        '''
        for node_range in self.ranges:
            node_range._sanity_check()
        
        try:
            while True:
                self.__partially_compact()
        except CompletelyCompact:
            return
            
    def __partially_compact(self):
        '''Try to make the NodeSelection a bit more compact.'''
        # todo: consider defining a canonic decomposition of a node selection to
        # node ranges. This will make a few things easier, like checking
        # equality.
        first, second = None, None
        for (r1, r2) in cute_iter_tools.orderless_combinations(self.ranges, 2):
            if r1.start in r2:
                second, first = r1, r2
                break
            elif r2.start in r1:
                first, second = r1, r2
                break
            else:
                pass
        if first is not None and second is not None:
            if second.end in first:
                pass
            else: # second.end not in first
                for current in second:
                    if current not in first:
                        break
                if current.parent is first.end:
                    self.ranges.remove(first)
                    new_range = NodeRange(start=first.start, end=second.end)
                else:
                    new_range = NodeRange(start=current, end=second.end)
                self.ranges.append(new_range)
              
            self.ranges.remove(second)
            return
        else:
            raise CompletelyCompact
    
    def __iter__(self):
        '''Iterate over the nodes that are members of this NodeSelection.'''
        for node_range in self.ranges:
            for node in node_range:
                yield node
    
    def __or__(self, other):
        '''Perform a union between two NodeSelections and return the result.'''
        assert isinstance(other, NodeSelection)
        return NodeSelection(self.ranges + other.ranges)
    
    def __ror__(self, other):
        return self.__or__(other)
    
    def __eq__(self, other):
        # Currently horribly inefficient
        if not isinstance(other, NodeSelection):
            return False # todo: should be NotImplemented?
        return set(self) == set(other)
        
    def __req__(self, other):
        return self.__eq__(other)
    
    def copy(self):
        '''Shallow-copy the NodeSelection.'''
        klass = type(self)
        return klass((node_range.copy() for node_range in self.ranges))
    
    __copy__ = copy
    
    def __hash__(self):
        # Defining __hash__ because there's __eq__ which makes the default
        # __hash__ disappear on Python 3.
        return id(self)
    
    


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines several functions that may be useful when working with queues.'''



import queue as queue_module

def dump(queue):
    '''
    Empty all pending items in a queue and return them in a list.
    
    Use only when no other processes/threads are reading from the queue.
    '''
    return list(iterate(queue))


def iterate(queue, block=False):
    '''Iterate over the items in the queue.'''
    
    while True:
        try:
            yield queue.get(block=block)
        except queue_module.Empty:
            raise StopIteration


def get_item(queue, i):
    '''
    Get an item from the queue by index number without removing any items.
    
    Note: This was designed for Queue.Queue. Don't try to use this, for
    example, on multiprocessing.Queue.
    '''
    with queue.mutex:
        return queue.queue[i]

def queue_as_list(queue):
    '''
    Get all the items in the queue as a list without removing them.
    
    Note: This was designed for Queue.Queue. Don't try to use this, for
    example, on multiprocessing.Queue.
    '''
    with queue.mutex:
        return list(queue.queue)


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the ObsoleteCruncherError exception.

See its documentation for more info.
'''

class ObsoleteCruncherError(BaseException):
    '''
    The cruncher that got this exception raised became obsolte.
    
    The cruncher is trying to do work, but in the meantime the main program
    decided that the work assigned to this cruncher is no longer wanted, and
    should therefore be stopped.
    '''
    # Inherits from BaseException because it is an exit exception


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
A collection of crunchers, which are workers that crunch the simulation.

A cruncher receives a state from the main program, and then it repeatedly
applies the step function of the simulation to produce more states. Those states
are then put in the cruncher's work_queue. They are then taken by the main
program when Project.sync_crunchers is called, and put into the tree.

The cruncher also receives a crunching profile from the main program. The
crunching profile specifes how far the cruncher should crunch the simulation,
and which arguments it should pass to the step function.

This package may define different crunchers which work in different ways, but
are to a certain extent interchangable. Different kinds of crunchers have
different advantages and disadvantges relatively to each other, and which
cruncher you should use for your project depends on the situation.

See the documentation for the different crunchers for more info.
'''

from .cruncher_thread import CruncherThread
from .cruncher_process import CruncherProcess




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
GarlicSim is a platform for writing, running and analyzing simulations.

It can handle any kind of simulation: Physics, game theory, epidemic spread,
electronics, etc.

Visit http://garlicsim.org for more info.

This package, called `garlicsim`, is the business logic. It is copyrighted to
Ram Rachum, 2009, and is distributed under the LGPL v2.1 License. The license
is included with this package as the file `lgpl2.1_license.txt`.

This licensing does not apply to `garlicsim_wx`, which is the associated GUI
package.

This program is intended for Python versions 2.5 and 2.6.
'''

from . import bootstrap
from . import general_misc
from .general_misc import monkeypatch_copyreg
from . import misc
from .asynchronous_crunching import Project
from .synchronous_crunching import simulate, list_simulate, iter_simulate

__all__ = ['Project', 'simulate', 'list_simulate', 'iter_simulate']

__version__ = '0.5.1'




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines different rounding options for binary search.'''

# Confirm that *_IF_BOTH options are used are used in all places that currently
# ~use them.

class Rounding(object):
    '''Base class for rounding options for binary search.'''


    
class BOTH(Rounding):
    '''
    Get a tuple (low, high) of the two items that surround the specified value.
    
    If there's an exact match, gives it twice in the tuple, i.e. (match, match).
    '''
    
class EXACT(Rounding):
    '''Get the item which has exactly the same value has the specified value.'''
    
class CLOSEST(Rounding):
    '''Get the item which has a value closest to the specified value.'''
    
class LOW(Rounding):
    '''
    Get the item with a value that is just below the specified value.
    
    i.e. the highest item which has a value lower or equal to the specified
    value.
    '''
    
class HIGH(Rounding):
    '''
    Get the item with a value that is just above the specified value.
    
    i.e. the lowest item which has a value higher or equal to the specified
    value.
    '''

class LOW_IF_BOTH(Rounding):
    '''
    Get the item with a value that is just below the specified value.
    
    i.e. the highest item which has a value lower or equal to the specified
    value.
    
    Before it returns the item, it checks if there also exists an item with a
    value *higher* than the specified value. If there isn't, it returns None.
    '''
    
class HIGH_IF_BOTH(Rounding):
    '''
    Get the item with a value that is just above the specified value.
    
    i.e. the lowest item which has a value higher or equal to the specified
    value.
    
    Before it returns the item, it checks if there also exists an item with a
    value *lower* than the specified value. If there isn't, it returns None.
    '''
    
class CLOSEST_IF_BOTH(Rounding):
    '''
    Get the item which has a value closest to the specified value.
    
    Before it returns the item, it checks if there also exists an item which is
    "on the other side" of the specified value. e.g. if the closest item is
    higher than the specified item, it will confirm that there exists an item
    *below* the specified value. (And vice versa.) If there isn't it returns
    None.
    '''
    
class LOW_OTHERWISE_HIGH(Rounding):
    '''
    Get the item with a value that is just below the specified value.
    
    i.e. the highest item which has a value lower or equal to the specified
    value.
    
    If there is no item below, give the one just above.
    '''
    
class HIGH_OTHERWISE_LOW(Rounding):
    '''
    Get the item with a value that is just above the specified value.
    
    i.e. the lowest item which has a value higher or equal to the specified
    value.
    
    If there is no item above, give the one just below.
    '''

roundings = (LOW, LOW_IF_BOTH, LOW_OTHERWISE_HIGH, HIGH, HIGH_IF_BOTH,
             HIGH_OTHERWISE_LOW, EXACT, CLOSEST, CLOSEST_IF_BOTH, BOTH)



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the HistoryBrowser class.

See its documentation for more info.
'''

import garlicsim.general_misc.binary_search as binary_search
import garlicsim.general_misc.queue_tools as queue_tools
import garlicsim.misc

__all__ = ["HistoryBrowser"]


get_state_clock = lambda state: state.clock


class HistoryBrowser(garlicsim.misc.BaseHistoryBrowser):
    '''
    A device for requesting information about the history of the simulation.
    
    A HistoryBrowser is a device for requesting information about the history of
    the simulation. It is intended to be used by CruncherThread in simulations
    that are history-dependent.
    
    This specific kind of history browser, defined in the synchronous_crunching
    package, is intended for synchronously-crunched simulations in which there
    are no worker processes/threads doing the crunching. Therefore, its job is
    quite simple; it recieves a path in its constructor and it handles all
    state requests from that path.
    '''
    def __init__(self, path, end_node=None):
        #todo: maybe not require path, just calculate from node?
        self.path = path
        '''
        This is the path, from which all states will be taken when requested.
        '''
        
        self.end_node = end_node
        '''
        An optional end node, in which the path ends.
        
        If not specified, it will be None, meaning that the path would go on
        until its natural end.
        
        If this option is specified, you will have to update the end_node of
        the history browser every time you use the step function. (That's
        because you've added a node to the tree, and that node should now be
        the end_node.)
        '''
     
    def get_last_state(self):
        '''Get the last state in the timeline. Identical to __getitem__(-1).'''
        return self.end_node.state if self.end_node else self[-1]
    
    def __getitem__(self, index):
        '''Get a state by its position in the timeline.'''
        assert isinstance(index, int)
        return self.path.__getitem__(index, end=self.end_node).state
    
    def get_state_by_monotonic_function(self, function, value,
                                        rounding=binary_search.CLOSEST):
        '''
        Get a state by specifying a measure function and a desired value.
        
        The function must be a monotonic rising function on the timeline.
        
        See documentation of binary_search.roundings for details about rounding
        options.
        '''
        assert issubclass(rounding, binary_search.Rounding)
        
        new_function = lambda node: function(node.state)
        result_in_nodes = self.path.get_node_by_monotonic_function \
                        (new_function, value, rounding, end_node=self.end_node)
        
        if rounding is binary_search.BOTH:
            result = [(node.state if node is not None else None) \
                      for node in result_in_nodes]
        else:
            result = result_in_nodes.state if result_in_nodes is not None \
                   else None
            
        return result
    
    def __len__(self):
        '''Get the length of the timeline in nodes.'''
        return self.path.__len__(end=self.end_node)
    


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
See documentation of class ReadWriteLock defined in this module.
'''

from . import original_read_write_lock

__all__ = ["ReadWriteLock"]

class ContextManager(object):
    def __init__(self, lock, acquire_func):
        self.lock = lock
        self.acquire_func = acquire_func
    def __enter__(self, *args, **kwargs):
        self.acquire_func()
    def __exit__(self, *args, **kwargs):
        self.lock.release()

class ReadWriteLock(original_read_write_lock.ReadWriteLock):
    '''
    A ReadWriteLock subclassed from a different ReadWriteLock class defined
    in the module original_read_write_lock.py, (See the documentation of the
    original class for more details.)
    
    This subclass adds two context managers, one for reading and one for
    writing.
    
    Usage:
    
    lock = ReadWriteLock()
    with lock.read:
        pass #perform read operations here
    with lock.write:
        pass #perform write operations here
    '''
    # todo: rename from acquireRead style to acquire_read style
    def __init__(self, *args, **kwargs):
        original_read_write_lock.ReadWriteLock.__init__(self, *args, **kwargs)
        self.read = ContextManager(self, self.acquireRead)
        self.write = ContextManager(self, self.acquireWrite)



# todo: use this in all the places where i `get` shit in garlicsim_wx, like
# get_background_brush

import functools

def cache(function):
    # todo: kwargs support
    # todo: try to be smart and figure out the function's signature, then be
    # able to understand that squared(x) is the same as sqaured(number=x).
    if hasattr(function, 'cache'): return function
    
    def cached(*args):
        
        try:
            return cached.cache[args]
        except KeyError:
            cached.cache[args] = value = function(*args)
            return value
            
    cached.cache = {} # weakref.WeakKeyDictionary()
    # todo: no weakref on this baby, be advised
    
    functools.update_wrapper(cached, function)
    
    return cached


class CachedType(type):
    @cache
    def __call__(cls, *args, **kwargs):
        # todo: should not use the generic cache function. need to analyze
        # signature of __init__. Possibly use the same args&kwargs grokker for
        # this and `cache`.
        return type.__call__(cls, *args, **kwargs)
    



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
A bootstrap module for garlicsim.

It checks all prerequisites are installed.
'''

import sys


if sys.version_info[0] <= 2:
    raise Exception('''This package requires Python 3.x. For Python 2.5+, use \
`garlicsim`, which you can find on PyPI.''')


def __check_prerequisites():
    '''
    Check that all modules required for garlicsim are installed.
    
    Returns a list of some imported modules: A reference to this list should be
    kept alive so to prevent the imported modules from being garbage-collected,
    which would cause Python to load them twice, which would needlessly increase
    startup time.
    '''
    
    modules = []
    
    class MissingModule(Exception):
        '''A required module is not found.'''
    
    def check_pkg_resources():
        try:
            import pkg_resources
            return [pkg_resources]
        except ImportError:
            raise MissingModule('''pkg_resources is required, but it's not \
currently installed on your system. It comes with either setuptools or \
Distribute, so please find either one of these on the internet and install \
it, then try again.''')
    
    
    checkers = [check_pkg_resources]
    
    for checker in checkers:
        modules += checker()
    
    return modules

__modules_list = __check_prerequisites()



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''This package provides a list of human names as `name_list`.'''

import pkg_resources

__all__ = ['name_list']

(male_raw, female_raw) = \
    [
        str(pkg_resources.resource_string(__name__, file_name)) for
        file_name in ['male.txt', 'female.txt']
    ]

name_list = male_raw.split(':') + female_raw.split(':')



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Persistent class.

See its documentation for more information.



Note: This module is still experimental

todo: need to raise an exception if we're getting pickled with
an old protocol?

todo: make it polite to other similar classes
'''

from .copy_modes import DontCopyPersistent
from garlicsim.general_misc import copy_tools


__all__ = ['Persistent']


class Persistent(object):
    '''
    Object that sometimes shouldn't really be duplicated.

    Say some plain object references a Persistent object. Then that plain object
    gets deepcopied with the DontCopyPersistent copy mode. The plain object will
    get deepcopied, but the Persistent object under it will not! The new copy of
    the plain object will refer to the same old copy of the Persistent object.
    
    This is useful for objects which are read-only and possibly heavy. You may
    use Persistent as a base class for these kinds of objects.
    
    When copying a Persistent, it is not really copied; The new "copy" is just
    the same object.
    
    Keep in mind that a Persistent is read-only. This means that starting from the
    first time that it is copied or put in a queue, it should not be changed.

    There is no mechanism that enforces that the user doesn't change the object,
    so the user must remember not to change it.
    
    Note: This class is still experimental.
    '''    
    
    def __deepcopy__(self, memo):
        '''
        Deepcopy the object. If DontCopyPersistent is given, only mock-copy.
        
        When this method receieves an instance of DontCopyPersistent as a memo
        dictionary, it will not actually deepcopy the object but only return a
        reference to the original object.
        '''
        if isinstance(memo, DontCopyPersistent):
            memo[id(self)] = self
            return self
        else:
            new_copy = copy_tools.deepcopy_as_simple_object(self, memo)
            return new_copy
    
        
    def __copy__(self):
        return self


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
A module that defines the State class.

See its documentation for more info.
'''

from garlicsim.general_misc import misc_tools

import garlicsim


class State(object):
    '''
    A state describes a world state in the world of the simulation.
    
    It contains information about a "frozen moment" in the simulation.

    All the information about the state of the simulation should be saved in
    attributes of the state object.

    When a state is created, a `.clock` attribute must be assigned to it,
    specifying what time it is in this state.

    A state object must always be picklable, as do all the attributes assigned
    to it.
    '''
    
    def __new__(cls, *args, **kwargs):
        if cls is State:
            raise garlicsim.misc.GarlicSimException('''You tried to \
create a State object whose class is `garlicsim.data_structures.State`. \
This is unallowed; This class should be used as a base class for State \
classes in simpacks.''')
        return super(type, cls).__new__(cls)
    
    
    def __repr__(self):
        '''
        Get a string representation of the state.
        
        Example output:
        <garlicsim.data_structures.State with clock 32.3 at 0x1c822d0>
        ''' 
        return '<%s %sat %s>' % \
               (
                   misc_tools.shorten_class_address(
                       self.__class__.__module__,
                       self.__class__.__name__
                       ),
                   'with clock %s ' % self.clock if hasattr(self, 'clock') \
                                      else '',
                   hex(id(self))
               )

    create_root = None
    create_messy_root = None
    # Just so we could easily check whether a subclass implemented these.
        



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines functions related to finding Python packages. See
documentation of get_packages for more info.

This module is hacky.
'''

import glob
import os
import types

def get_packages(root, include_self=False, recursive=False, self_in_name=True):
    '''
    Find all sub-packages.
    
    `root` may be a module, package, or a path.
    # todo: module? really?
    '''
    
    if isinstance(root, types.ModuleType):
        root_module = root
        root_path = os.path.dirname(root_module.__file__)
    elif isinstance(root, str):
        root_path = os.path.abspath(root)
        # Not making root_module, it might not be imported.
    
    root_module_name = os.path.split(root_path)[1]

    ######################################################
    
    result = []
    
    if include_self:
        result.append('')
        
    for entry in os.listdir(root_path):
        full_path = os.path.join(root_path, entry)
        if is_package(full_path):
            if recursive:
                result += ['.' + thing for thing in 
                           get_packages(full_path, include_self=True,
                                        recursive=True)]
            else:
                result.append('.' + entry)
    
    if self_in_name:
        return [(root_module_name + thing) for thing in result]
    else:
        return result
    

def get_packages_and_modules_filenames(root, recursive=False):
    '''
    Find the filenames of all of the packages and modules inside the package.
    
    `root` may be a module, package, or a path.
    todo: module? really?
    todo: needs testing
    '''
        
    if isinstance(root, types.ModuleType):
        root_module = root
        root_path = os.path.dirname(root_module.__file__)
    elif isinstance(root, str):
        root_path = os.path.abspath(root)
        # Not making root_module, it might not be imported.
    
    root_module_name = os.path.split(root_path)[1]

    ######################################################
    
    result = []
        
    for entry in os.listdir(root_path):
        
        full_path = os.path.join(root_path, entry)
        
        if is_module(full_path):
            result.append(entry)
            continue
            
        elif is_package(full_path):
            result.append(entry)
            if recursive:
                inner_results = get_packages_and_modules_filenames(
                    full_path,
                    recursive=True
                )
                result += [os.path.join(entry, thing) for thing in
                           inner_results]
    
    return [os.path.join(os.path.dirname(full_path), entry) for entry in result]

def is_package(path):
    '''Is the given path a Python package?'''
    return os.path.isdir(path) and \
           glob.glob(os.path.join(path, '__init__.*'))

def is_module(path):
    '''Is the given path a Python single-file module?'''
    extension = os.path.splitext(path)[1]
    return extension.lower() in ['.py', '.pyc', '.pyo', '.pyw']




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines the BinarySearchProfile class.

See its documentation for more info.
'''


from .roundings import (Rounding, roundings, LOW, LOW_IF_BOTH,
                        LOW_OTHERWISE_HIGH, HIGH, HIGH_IF_BOTH,
                        HIGH_OTHERWISE_LOW, EXACT, CLOSEST, CLOSEST_IF_BOTH,
                        BOTH)

from .functions import (binary_search, binary_search_by_index,
                        make_both_data_into_preferred_rounding)
        
        
class BinarySearchProfile(object):
    '''
    A profile of binary search results.
    
    A binary search profile allows to access all kinds of aspects of the results
    of a binary search, while not having to execute the search more than one
    time.
    '''
    
    def __init__(self, sequence, function, value, both=None):
        '''
        Constructor for BinarySearchProfile.
        
        `sequence` is the sequence through which the search is made. `function`
        is a monotonically rising function on the sequence. `value` is the
        wanted value.
        
        In the `both` argument you may put binary search results (with the BOTH
        rounding option.) This will prevent the constructor from performing the
        search itself. It will use the results you provided when giving its
        analysis.
        '''

        if both is None:
            both = binary_search(sequence, function, value, BOTH)
        
        self.results = {}
        '''
        `results` is a dict from rounding options to results that were obtained
        using each function.
        '''
        
        for rounding in roundings:
            self.results[rounding] = \
                make_both_data_into_preferred_rounding(both, function, value,
                                                       rounding)
        none_count = list(both).count(None)
        
        self.all_empty = (none_count == 2)
        '''Flag saying whether the sequence is completely empty.'''
        
        self.one_side_empty = (none_count == 1)
        '''Flag saying whether the value is outside the sequence's scope.'''
        
        self.is_surrounded = (none_count == 0)
        '''Flag saying whether the value is inside the sequence's scope.'''
            
        self.had_to_compromise = {
            LOW_OTHERWISE_HIGH:
                self.results[LOW_OTHERWISE_HIGH] is not self.results[LOW],
            HIGH_OTHERWISE_LOW:
                self.results[HIGH_OTHERWISE_LOW] is not self.results[HIGH],
        }
        '''
        Dictionary from "otherwise"-style roundings to bool.
        
        What this means is whether the "otherwise" route was taken. See
        documentation of LOW_OTHERWISE_HIGH for more info.
        '''
        
        self.got_none_because_no_item_on_other_side = {
            LOW_IF_BOTH:
                self.results[LOW_IF_BOTH] is not self.results[LOW],
            HIGH_IF_BOTH:
                self.results[HIGH_IF_BOTH] is not self.results[HIGH],
            CLOSEST_IF_BOTH:
                self.results[CLOSEST_IF_BOTH] is not self.results[CLOSEST],
        }
        '''
        Dictionary from "if both"-style roundings to bool.
        
        What this means is whether the result was none because the BOTH result
        wasn't full. See documentation of LOW_IF_BOTH for more info.
        '''
        
        for d in [self.had_to_compromise,
                  self.got_none_because_no_item_on_other_side]:
            
            for rounding in roundings:
                if rounding not in d:
                    d[rounding] = None
        


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
A module that defines the Node class and the related NodeError exception.

See documentation of Node for more information.
'''

from garlicsim.general_misc.infinity import Infinity
from garlicsim.general_misc import misc_tools

from garlicsim.misc import GarlicSimException

from .state import State
from .tree_member import TreeMember

# We are doing `from block import Block` in the bottom of the file.
# We are doing `from path import Path` in the bottom of the file.


__all__ = ["Node", "NodeError"]


class NodeError(GarlicSimException):
    '''Node-related Exception.'''

class NodeLookupError(NodeError, LookupError):
    '''A node-related lookup was requested but no result was found.'''
    
    
class Node(TreeMember):
    '''
    Nodes are used to organize states in a tree.
    
    A node encapsulates a state with the attribute `.state`. 
    
    Most nodes are untouched, a.k.a. natural, but some nodes are touched.
    A touched node is a node whose state was not formed naturally by a
    simulation step: It was created by the user, either from scratch or based
    on another state.
    '''
    # todo: Maybe node should not reference tree?
    
    def __init__(self, tree, state, parent=None, step_profile=None,
                 touched=False):
        '''
        Construct the node.
        
        `tree` is the tree in which this node resides. `state` is the state it
        should contain. `parent` is its parent node in the tree, which may be
        None for a root. `step_profile` is the step profile with which the state
        was crunched, which may be None for a state that was created from
        scratch. `touched` is whether the state was modified/created from
        scratch, in contrast to having been produced by crunching.
        '''
        
        self.tree = tree
        '''The tree in which this node resides.'''
        
        self.state = state
        '''The state contained in the node.'''
        
        self.parent = parent
        '''The parent node of this node.'''
        
        self.step_profile = step_profile
        '''
        The step options profile with which the contained state was created.
        
        For an untouched node, this must be a real StepProfile, even an empty
        one. Only a touched node which was created from scratch should have
        None for its step profile.
        '''
        
        self.touched = touched
        '''Says whether the node is a touched node.'''
        
        self.block = None
        '''
        A node may be a member of a block. See class Block for more details.
        '''

        self.children = []
        '''
        A list of:
            1. Nodes whose states were produced by simulation from this node.
            2. Nodes who were "created by editing" from one of the nodes in the
            aforementioned set.
        '''

        self.derived_nodes = []
        '''
        List of nodes who were created by editing from this node.
        
        These nodes should have the same parent as this node.
        '''

        self.still_in_editing = False
        '''
        A flag that is raised for a node which is "still in editing".
        
        This means that its state is still being edited and was not yet
        finalized, thus no crunching should be made from the node until it is
        finalized.
        '''
        
        self.ends = []
        '''
        The ends whose parent is this node.
        
        This means, world ends that were arrived to on a timeline terminating
        with this node.
        '''
  
        
    def __len__(self):
        '''Just return 1. This is useful because of blocks.'''
        return 1

    
    def finalize(self):
        '''
        Finalize the node, assuming it's in currectly in editing mode.
        
        Before an edited node is finalized, it cannot be crunched from and
        cannot have children. (i.e. nodes that follow it in time.) After getting
        finalized, it may be crunched from and be assigned children.
        '''
        if self.still_in_editing is False:
            if self.touched:
                message = '''You tried to finalize a touched node, but it has \
already been finalized.'''
            else: # self.touched is False
                message = '''You tried to finalize an untouched node. \
Untouched nodes can't be edited, so they have no concept of being finalized.'''
            raise NodeError(message)
        
        self.still_in_editing = False

        
    def soft_get_block(self):
        '''
        If this node is a member of a block, return the block.
        
        Otherwise, return the node itself.
        '''
        return self.block or self

    
    def make_containing_path(self):
        '''
        Create a path that contains this node.
        
        There may be multiple different paths that contain this node. This will
        return the one which points to the newest possible forks.
        
        Returns the path.
        '''
        
        path = self.make_past_path()
        
        path.get_last_node()
        # Calling this will make the path notice the forks in the nodes beyond
        # this node and put them in its `decisions` dict.
        
        return path
    
        
    def all_possible_paths(self):
        '''
        Get a list of all possible paths that contain this node.
        
        Note: There may be paths that contain this node which will not be
        identical to one of the paths given here, because these other paths
        may specify decisions that are not even on the same root as these
        paths.
        '''
        #todo: possibly add `reversed` option
        past_path = self.make_past_path()
        paths = []
        fork = None
        for thing in past_path.iterate_blockwise(start=self):
            real_thing = thing[-1] if isinstance(thing, Block) else thing
            if len(real_thing.children):
                fork = real_thing
                break
        
        if fork:
            for kid in fork.children:
                paths += kid.all_possible_paths()
            return paths
        else: # fork is None and real_thing is the final node of the path
            # In this case there are no forks after our node, we just return
            # the past_path which we have driven to its end. (Not that it has
            # any forks to decide on anyway.)
            return [past_path]
    
    def make_past_path(self):
        '''
        Create a path that contains this node.
        
        There may be multiple different paths that contain this node. This will
        return a path that doesn't specify any decisions after this node.
        '''
        path = Path(self.tree)

        current = self
        while True:
            if current.block is not None:
                current = current.block[0]
            parent = current.parent
            if parent is None:
                path.root = current
                break
            if len(parent.children) > 1:
                path.decisions[parent] = current
            current = parent

        return path

    def get_all_leaves(self, max_nodes_distance=None, max_clock_distance=None):
        '''
        Get all leaves that are descendents of this node.
        
        Only leaves with a distance of at most `max_nodes_distance` in nodes or
        `max_clock_distance` in clock are returned. (Note this is an OR
        relation between the two condintions)
        
        Returns a dict of the form:
        
        {
            leaf1: {
                'nodes_distance': nodes_distance1,
                'clock_distance': clock_distance1,
            },            
            leaf2: {
                'nodes_distance': nodes_distance2,
                'clock_distance': clock_distance2,
            },
            # ...
        }
            
        '''
        if max_nodes_distance is None:
            max_nodes_distance = Infinity
        if max_clock_distance is None:
            max_clock_distance = Infinity
                    
        nodes = {self: {"nodes_distance": 0, "clock_distance": 0}}
        leaves = {}

        while nodes:
            item = nodes.popitem()
            node = item[0]
            nodes_distance = item[1]["nodes_distance"]
            clock_distance = item[1]["clock_distance"]
            
            if nodes_distance > max_nodes_distance and \
               clock_distance > max_clock_distance:
                continue
            
            kids = node.children
            
            if not kids:
                # We have a leaf!
                leaves[node] = {
                    "nodes_distance": nodes_distance,
                    "clock_distance": clock_distance,
                }
                continue
            
            if (node.block is None) or node.is_last_on_block():
                for kid in kids:
                    nodes[kid] = {
                        "nodes_distance": nodes_distance + 1,
                        "clock_distance": kid.state.clock - self.state.clock,
                    }
                continue
            else:
                block = node.block
                index = block.index(node)
                rest_of_block = (len(block) - index - 1)
                
                # We know the node isn't the last on the block, because we
                # checked for that before.

                last = block[-1]
                nodes[last] = {
                    "nodes_distance": nodes_distance + rest_of_block,
                    "clock_distance": last.state.clock - self.state.clock,
                }
                continue
            
        return leaves

    
    def get_ancestor(self, generations=1, round=False):
        '''
        Get an ancestor of this node.
        
        `generations` specifies the number of generation that the returned
        ancestor should be above the current node. `round` determines how this
        method will behave if it was asked for too many generations back, and
        not enough existed. If `round` is True, it will return the root. If
        `round` is False, it will raise a LookupError.
        '''

        assert generations >= 0
        if generations == 0:
            return self
        if generations == 1:
            if self.parent:
                return self.parent
            else: # self.parent is None
                if round:
                    return self
                else: # round is False
                    raise NodeLookupError('''You asked for the node's parent, \
but it's a root.''')                
                
        block = self.block
        if block:
            our_index = block.index(self)
            wanted_index = our_index - generations
            if wanted_index >= 0:
                return block[wanted_index]
            else: # wanted index < 0
                first_node = block[0]
                parent_of_first = first_node.parent
                if parent_of_first is None:
                    if round:
                        return first_node
                    else: # round is False
                        raise NodeLookupError('''You asked for too many \
generations back. This node's ancestry line doesn't go back that far.''')
                
                return parent_of_first.get_ancestor(- wanted_index - 1, round=round)
        
        assert self.block is None
        if self.parent:
            return self.parent.get_ancestor(generations - 1, round)
        else:
            if round:
                return self
            else: # round is False
                raise NodeLookupError('''You asked for too many generations \
back. This node's ancestry line doesn't go back that far.''')

            
    def get_root(self):
        '''
        Get the root of this node.
        
        This means the node which is the parent of the parent of the parent
        of... the parent of this node.
        '''
        lowest = self.block[0] if self.block else self
        while lowest.parent is not None:
            lowest = lowest.parent
            if lowest.block:
                lowest = lowest.block[0]
        return lowest
    
    def is_last_on_block(self):
        '''Return whether the node the last one on its block.'''
        return self.block and (self.block.index(self) == len(self.block) - 1)
    
    def is_first_on_block(self):
        '''Return whether the node the first one on its block.'''
        return self.block and (self.block.index(self) == 0)
    
    def is_overlapping(self, tree_member):
        '''
        Return whether this node overlaps with the given tree_member.
        
        `tree_member` may be a node, in which case overlapping means being the
        same node. `tree_member` can also be a block, in which case overlapping
        means this node is contained in the block.
        '''
        if tree_member is None: return False
        if isinstance(tree_member, Node):
            return (self is tree_member)
        else:
            assert isinstance(tree_member, Block)
            return (self in tree_member)
    
    def __repr__(self):
        '''
        Get a string representation of the node.
        
        Example output:
        <garlicsim.data_structures.Node with clock 6.5, untouched, belongs
        to a block, crunched with StepProfile(t=0.1), at 0x1ffde70>
        '''
        return '<%s%s, %s%s%s, %s, %sat %s>' % \
            (
                misc_tools.shorten_class_address(
                    self.__class__.__module__,
                    self.__class__.__name__
                    ),
                ' with clock %s' % self.state.clock if hasattr(self.state, 'clock') else '',
                'root, ' if (self.parent is None) else '',
                'leaf, ' if (len(self.children) == 0) else '',
                'touched' if self.touched else 'untouched',
                'belongs to a block' if self.block else 'blockless',
                'crunched with %s, ' % self.step_profile if self.step_profile else '',
                hex(id(self))
            )

from .path import Path
from .block import Block





# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the StepCopy class.

See its documentation for more information.
'''

from garlicsim.general_misc.copy_mode import CopyMode
from garlicsim.misc.persistent import DontCopyPersistent

class StepCopy(DontCopyPersistent, CopyMode):
    '''
    A copy mode used in a step function to generate the next state.
    
    A popular design pattern in step functions is to deepcopy the old state,
    modify it, and then return it as the new state. When this is done, you
    should pass StepCopy() into the deepcopy function as a memo. This assures
    that certain objects get copied in the right way for this context.
    '''



import time
import sys
import multiprocessing

# import logging
# logger = multiprocessing.log_to_stderr()
# logger.setLevel(logging.DEBUG)


STOPPED = 'STOP'

def dump(queue):
    result = []
    
    for thing in iter(queue.get, STOPPED):
        if not isinstance(thing, list):
            print('we got something we should not have: %s' % thing)
        result.append(thing)
    return result



class Process(multiprocessing.Process):
    def __init__(self, x):
        multiprocessing.Process.__init__(self)
        self.q = multiprocessing.Queue()
        self.event = x
    def run(self):
        for i in range(100):
            self.q.put([list(range(20)) for j in range(10)])
        self.q.put(STOPPED)
        self.event.set()


if __name__ == '__main__':
    x = multiprocessing.Event()
    p = Process(x)
    p.start()
    x.wait()
    l = dump(p.q)
    print(len(l))
    p.join()


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the DontCopyPersistent class.

See its documentation for more information.
'''

from garlicsim.general_misc.copy_mode import CopyMode

class DontCopyPersistent(CopyMode):
    '''
    A copy mode under which Persistent objects aren't actually copied.
    
    When a Persistent is getting deepcopied with this mode, a reference to the
    original object would be returned instead of actually deepcopying it.
    
    Keep in mind that if the Persistent holds reference to additional objects,
    they too will not really be copied under this mode.
    '''




import inspect

class E(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)
        print((inspect.trace()))
        print((inspect.stack()))
        raise self
    

assert False, E("asdf")


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This package defines the SimpackGrokker and the Settings classes.

See their documentation for more info.
'''

from .settings import Settings
from .simpack_grokker import SimpackGrokker



from .state import *

import garlicsim


# FORCE_CRUNCHER = garlicsim.asynchronous_crunching.crunchers.CruncherThread

# A cruncher that this simpack insists on using.  
# 
# This is useful because some simpacks can't be used with certain kinds of
# crunchers.



# DETERMINISM_FUNCTION = garlicsim.misc.simpack_grokker.misc.default_determinism_function

# Function that takes a step profile and says whether its deterministic.
#
# What this function says is, "If you do a simulation using this step profile,
# then you will have a deterministic simulation." (Or undeterministic, depends
# on the step profile.)
#
# This is useful because it allows garlicsim to detect if a simulation has
# reached a repititive state, so it can stop the crunching right there and avoid
# wasting resources. (05.15.2010 - Still not implemented, sorry.)
#
# Note that this function does not return True or False: It returns a
# `DeterminismSetting` class. For details about those, see documentation in
# garlicsim.misc.settings_constants.settings.
#
# The function will return None if it's unknown whether the step profile is
# deterministic.



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines various tools related to importing.
'''

import sys
import os.path
import imp
from garlicsim.general_misc import package_finder

def import_by_path(path, name=None):
    '''
    Import module/package by path.
    
    You may specify a name: This is helpful only if it's an hierarchical name,
    i.e. a name with dots like "orange.claw.hammer". This will become the
    imported module's __name__ attribute. Otherwise only the short name,
    "hammer", will be used, which might cause problems in some cases. (Like
    when using multiprocessing.)
    '''
    short_name = os.path.splitext(os.path.split(path)[1])[0]
    if name is None: name = short_name
    path_to_dir = os.path.dirname(path)
    my_file = None
    try:
        (my_file, pathname, description) = \
            imp.find_module(short_name, [path_to_dir])
        module = imp.load_module(name, my_file, pathname, description)
    finally:
        if my_file is not None:
            my_file.close()
        
    return module
    

def import_all(package, exclude='__init__', silent_fail=False):
    '''
    Import all the modules and packages that live inside the given package.
    
    This is not recursive. Modules and packages defined inside a subpackage
    will not be imported (of course, that subpackage itself may import them
    anyway.)
    
    You may specify a module/package to exclude, which is by default
    `__init__`.
    
    Returns a list with all the imported modules and packages.
    
    todo: only tested with __init__ passed in
    '''
    
    paths = package_finder.get_packages_and_modules_filenames(package)
    
    names = {}
    for path in paths:
        name = os.path.splitext(os.path.split(path)[1])[0]
        if name == exclude:
            continue
        full_name = package.__name__ + '.' + name
        names[path] = full_name
        
    d = {}
    
    for (path, name) in list(names.items()):
        try:
            d[name] = import_by_path(path, name)
        except Exception:
            if not silent_fail:
                raise
    
    return d
    
    
    
def import_if_exists(module_name):
    '''Import module by name, only if it exists, otherwise return None.'''    
    if '.' in module_name:
        package_name, submodule_name = module_name.rsplit('.', 1)
        package = __import__(package_name, fromlist=[''])
        package_path = package.__path__
        try:
            imp.find_module(submodule_name, package_path)
        except ImportError:
            return None
    else: # '.' not in module_name
        try:
            imp.find_module(module_name)
        except ImportError:
            return None

    # Not actually using the result of `imp.find_module`, just want to know that
    # it worked and the module exists. We'll let the conventional `__import__`
    # find the module again, assuming its finding procedure will work exactly
    # the same as imp's.
        
    return __import__(module_name, fromlist=[''])
    
    
    
    
    




# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

import types

discardables = ['__builtins__']

wrapped_modules = {}

def module_wrapper_factory(module):
    '''
    Given a module, returns a ModuleWrapper which is almost identical to the
    module, with the difference being that it can be pickled.
    '''
    if module in wrapped_modules:
        return wrapped_modules[module]
    else:
        module_wrapper = ModuleWrapper(module)
        return module_wrapper

class ModuleWrapper(object):
    def __init__(self, module):
        wrapped_modules[module] = self
        self.__dict__ = dict(module.__dict__)
        for name, thing in list(self.__dict__.items()):
            '''
            Note this is a weak form of recursive scanning
            '''
            if name in discardables:
                self.__dict__[name] = "Missing item, string representation:" +\
                                      str(thing)
                continue
            if isinstance(thing, types.ModuleType):
                self.__dict__[name] = "Missing module " + thing.__name__
                continue
                
                
if __name__ == "__main__":
    import pickle
    def test(module):
        try:
            pickle.dumps(ModuleWrapper(module))
            return True
        except Exception:
            return False
        
    import garlicsim_wx.simulation_packages.life as life
    print((test(life)))
    pickle.dumps(ModuleWrapper(life))


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines data structures that are used for saving simulation data.

States are used to save a single world state of the simulation. Nodes are used
to put states in chronological order inside a Tree. Paths are used to indicate
specific timelines inside a tree. Blocks, which are not as critical as the
other types of objects, are used to optimize access to long successions of
nodes.
'''



from .tree_member import TreeMember
from .state import State
from .tree import Tree, TreeError
from .path import Path, PathError, PathLookupError, PathOutOfRangeError
from .node import Node, NodeError
from .block import Block, BlockError
from .end import End

from .node_range import NodeRange
from .node_selection import NodeSelection


__all__ = ['TreeMember', 'State', 'Tree', 'Path', 'Node', 'Block', 'End',
           'NodeRange', 'NodeSelection'] + \
          ['BlockError', 'PathError', 'PathLookupError', 'PathOutOfRangeError',
            'TreeError', 'NodeError']


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines several functions that may be useful when working with dicts.'''

import copy

def get_list(d, container):
    '''Get a list of the values in the dict whose keys are in `container`.'''
    return [value for (key, value) in d.items() if (key in container)]

def fancy_string(d, indent=0):
    '''Show a dict as a string, slightly nicer than dict.__repr__.'''

    small_space = ' ' * indent
    
    big_space = ' ' * (indent + 4)
    
    huge_space = ' ' * (indent + 8)
    
    def show(thing, indent=0):
        space = ' ' * indent
        enter_then_space = '\n' + space
        return repr(thing).replace('\n', enter_then_space)
    
    temp1 = (
        (big_space + repr(key) + ':\n' + huge_space + show(value, indent + 8))
        for(key, value) in list(d.items()))
    
    temp2 = small_space + '{\n' + ',\n'.join(temp1) + '\n' + small_space +'}'
    
    return temp2
    
    
    


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the CrossProcessPersistent class.

See its documentation for more information.



Note: This module is still experimental.

todo: need to lock library to avoid thread trouble?
'''


import uuid
import weakref
import colorsys

from .copy_modes import DontCopyPersistent
from garlicsim.general_misc import misc_tools
from garlicsim.general_misc import copy_tools

from .persistent import Persistent
from .personality import Personality


library = weakref.WeakValueDictionary()


class UuidToken(object):
    '''Token which contains a uuid with its attribute `.uuid`'''
    def __init__(self, uuid):
        self.uuid = uuid

        
class CrossProcessPersistent(Persistent):
    '''
    Object that sometimes shouldn't really be duplicated.

    Say some plain object references a CrossProcessPersistent object. Then that
    plain object gets deepcopied with the DontCopyPersistent copy mode. The
    plain object will get deepcopied, but the CrossProcessPersistent object
    under it will not! The new copy of the plain object will refer to the same
    old copy of the CrossProcessPersistent object.
    
    This is useful for objects which are read-only and possibly heavy. You may
    use Persistent as a base class for these kinds of objects.
    
    When copying a CrossProcessPersistent, it is not really copied; The new
    "copy" is just the same object. 
    
    Keep in mind that a CrossProcessPersistent is read-only. This means that
    starting from the first time that it is copied or put in a queue, it should
    not be changed.

    There is no mechanism that enforces that the user doesn't change the object,
    so the user must remember not to change it.
    
    What this class adds over Persistent, is that when a CrossProcessPersistent
    is passed around between processes in queues, each process retains only one
    copy of it.
    
    Note: This class is still experimental.
    '''
    def __new__(cls, *args, **kwargs):
        
        # Here we need to check in what context __new__ was called.
        # There are two options:
        #     1. The object is being created.
        #     2. The object is being unpickled.
        # We check whether we are getting a uuid token. If we are, it's
        # unpickling. If we don't, it's creation.
        
        if len(args)==1 and len(kwargs)==0 and isinstance(args[0], UuidToken):
            received_uuid = args[0].uuid
        else:
            received_uuid = None
            
        if received_uuid: # The object is being unpickled
            thing = library.pop(received_uuid, None)
            if thing:
                thing._CrossProcessPersistent__skip_setstate = True
                return thing
            else: # This object does not exist in our library yet; Let's add it
                thing = super(CrossProcessPersistent, cls).__new__(cls)
                thing._CrossProcessPersistent__uuid = received_uuid
                library[received_uuid] = thing
                return thing
                
        else: # The object is being created
            thing = super(CrossProcessPersistent, cls).__new__(cls)
            new_uuid = uuid.uuid4()
            thing._CrossProcessPersistent__uuid = new_uuid
            library[new_uuid] = thing
            return thing

        
    def __getstate__(self):
        my_dict = dict(self.__dict__)
        del my_dict["_CrossProcessPersistent__uuid"]
        return my_dict

    
    def __getnewargs__(self):
        return (UuidToken(self._CrossProcessPersistent__uuid),)

    
    def __setstate__(self, state):
        if self.__dict__.pop("_CrossProcessPersistent__skip_setstate", None):
            return
        else:
            self.__dict__.update(state)

            
    def __deepcopy__(self, memo):
        '''
        Deepcopy the object. If DontCopyPersistent is given, only mock-copy.
        
        When this method receieves an instance of DontCopyPersistent as a memo
        dictionary, it will not actually deepcopy the object but only return a
        reference to the original object.
        '''
        if isinstance(memo, DontCopyPersistent):
            memo[id(self)] = self
            return self
        else:
            new_copy = copy_tools.deepcopy_as_simple_object(self, memo)
            new_copy._Persistent__uuid = uuid.uuid4()
            try:
                del self.personality
            except AttributeError:
                pass
            return new_copy

        
    personality = misc_tools.LazilyEvaluatedConstantProperty(Personality)





# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines various tools for working with simpacks.'''


from garlicsim.general_misc import misc_tools
from garlicsim.general_misc import caching

import garlicsim.misc.simpack_grokker


def get_from_state(state):
    '''Find the simpack that a state class (or a state instance) belong to.'''
    state_class = state if isinstance(state, type) else type(state)
    return _get_from_state_class(state_class)
    

@caching.cache
def _get_from_state_class(state_class):
    '''
    Find the simpack that a state class belong to.
    
    Internal use.
    '''
    assert state_class.__name__ == 'State' # remove this limitation
    original_module_name = state_class.__module__
    short_address = misc_tools.shorten_class_address(original_module_name,
                                                     state_class.__name__)
    simpack_name = '.'.join(short_address.split('.')[:-1])
    simpack = __import__(simpack_name, fromlist=[''])
    try:
        garlicsim.misc.simpack_grokker.SimpackGrokker(simpack)
        # Not saving the reference: But it'll get cached because SimpackGrokker
        # is a CachedType.
    except garlicsim.misc.InvalidSimpack:
        raise garlicsim.misc.GarlicSimException('''Could not guess \
simpack correctly from state object.''')
    return simpack
    
    


#!/usr/bin/env python

# Copyright 2009 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Setuptools setup file for garlicsim_py3.'''

import os
import sys
import setuptools


if sys.version_info[0] <= 2:
    raise Exception('''This package requires Python 3.x. For Python 2.5+, use \
`garlicsim`, which you can find on PyPI.''')


def get_packages():
    return ['garlicsim.' + p for p in
            setuptools.find_packages('./garlicsim')] + \
           ['garlicsim']

my_long_description = \
'''\
GarlicSim is a platform for writing, running and analyzing simulations. It can
handle any kind of simulation: Physics, game theory, epidemic spread,
electronics, etc.

Visit http://garlicsim.org for more info.
'''

my_classifiers = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.1',
    'Topic :: Scientific/Engineering',
]


setuptools.setup(
    name='garlicsim_py3',
    version='0.5.1',
    description='Pythonic framework for working with simulations',
    author='Ram Rachum',
    author_email='cool-rr@cool-rr.com',
    url='http://garlicsim.org',
    packages=get_packages(),
    scripts=['garlicsim/scripts/start_simpack.py'],
    license="LGPL v2.1",
    long_description = my_long_description,
    classifiers = my_classifiers,
    include_package_data = True,
    zip_safe=False,
)
    



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines miscellaneous objects for `simpack_grokker`.'''

def default_determinism_function(step_profile):
    '''
    The default determinism function.
    
    Just returns None, meaning we have no idea whether the step profile is
    deterministic.
    '''
    return None


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''Defines a collection of exceptions.'''

from garlicsim.general_misc.exceptions import CuteException

class GarlicSimException(CuteException):
    '''GarlicSim-related exception.'''

class GarlicSimWarning(Warning):
    '''GarlicSim-related warning.'''
    

class InvalidSimpack(GarlicSimException):
    '''Trying to load an invalid simpack.'''

class SimpackError(GarlicSimException):
    '''A simpack behaved unexpectedly.'''
    
class WorldEnd(GarlicSimException):
    '''The simulation has ended.'''

    
    


# This is your `wx` package. It's a sub-package inside your simpack, and like
# most things in your simpack, completely optional. You will only want to fill
# it if you are interested in running your simulation in `garlicsim_wx`.
# Actually, even if you don't fill it in, your simulation will run in
# `garlicsim_wx`-- Just not very prettily.
#
# To make your simulation be prettier, you should start with filling in a state
# viewer. That's in the `wx/widgets/state_viewer.py` module. You may also make a
# state creation dialog, in `wx/widgets/state_creation_dialog`.
#
# When you're done, don't forget to visit `wx/settings.py` to link to the new
# classes you created.



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the StepProfile class.

See its documentation for more information.
'''


import copy

__all__ = ['StepProfile']


class StepProfile(object): # todo: use CachedType?
    '''
    A profile of *args and **kwargs to be used with a step function.
    
    Usage:
    
    step_profile = StepProfile(34, "meow", width=60)

    step(state, *step_profile.args, **step_profile.kwargs)
    # is equivalent to
    step(state, 34, "meow", width=60)
    '''
    # todo: perhaps this should be based on an ArgumentsProfile after all?
    # In __repr__ and stuff we'll just check self's class. How does Python
    # do it when you subclass its builtin types?
    
    def __init__(self, *args, **kwargs):
        
        # Perhaps we were passed a StepProfile object instead of args
        # and kwargs? If so load that one, cause we're all cool and nice.
        candidate = None
        if len(args) == 1 and len(kwargs) == 0:
            candidate = args[0]
        if len(args) == 0 and len(kwargs) == 1 and \
           ('step_profile' in kwargs):
            candidate = kwargs['step_profile']
        
        if isinstance(candidate, StepProfile):
            self.__load_from(candidate)
            return
        
        self.args = tuple(args)
        self.kwargs = kwargs  # todo: Get some frozendict class for this
        
        
    def __load_from(self, profile):
        '''
        Take another step options profile and load its arguments into this one.
        '''
        self.args = copy.copy(profile.args)
        self.kwargs = copy.copy(profile.kwargs)
        
    def __repr__(self):
        '''
        Get a string representation of the step profile.
        
        Example output:
        StepProfile('billinear', t=7)
        '''
        args_string = ', '.join([repr(thing) for thing in self.args])
        kwargs_string = ', '.join([str(key)+'='+repr(value) for \
                                   (key, value) in list(self.kwargs.items())])
        strings = [thing for thing in [args_string, kwargs_string] if \
                   thing]        
        big_string = ', '.join(strings)
        return 'StepProfile(%s)' % big_string
    
    def __eq__(self, other):
        return isinstance(other, StepProfile) and \
               self.args == other.args and self.kwargs == other.kwargs
    
    def __hash__(self):
        # Defining __hash__ because there's __eq__ which makes the default
        # __hash__ disappear on Python 3.
        return id(self)

    def __ne__(self, other):
        return not self.__eq__(other)


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''This module monkeypatches the pickling dispatch table using `copyreg`.'''

# todo: alters global state, yuck! Maybe check before if it's already set to
# something?

import sys
import copyreg
import types
import builtins
import pickle

###############################################################################

def reduce_method(method):
    '''Reducer for methods.'''
    # todo: On Python 2.x this also works for unbound methods for some
    # mysterious reason. Unbound methods are still of the type `MethodType`, and
    # their `.im_self`/`.__self__` is None, so how does it work? Possibly
    # `pickle` in Python 2.x actually knows how to pickle unbound methods, and
    # doesn't really use this?
    #
    # In any case, in Python 3.x this works only for bound methods, and unbound
    # methods can't be pickled, which is an issue.
    return (getattr, (method.__self__, method.__func__.__name__))
    

copyreg.pickle(types.MethodType, reduce_method)

###############################################################################

def __import__(*args, **kwargs):
    '''Wrapper for the builtin `__import__`'''
    # todo: This is needed when debugging in Wing, cause Wing replaces
    # `__import__` with its own. This feels bad.
    return builtins.__import__(*args, **kwargs)

def reduce_module(module):
    '''Reducer for modules.'''
    return (__import__, (module.__name__, {}, {}, [''])) # fromlist cruft

copyreg.pickle(types.ModuleType, reduce_module)



# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
This module defines the Job class.

See its documentation for more info.
'''

__all__ = ['Job']

class Job(object):
    '''
    A job of crunching the simulation from a given node.
        
    A job specifies a node and a crunching profile. It means we should crunch
    from `node` according to the cruncing profile.
    '''
    # todo: should there be other helpful methods here?
    
    def __init__(self, node, crunching_profile):
        
        self.node = node
        '''The node from which we need to crunch.'''
        
        self.crunching_profile = crunching_profile
        '''The crunching profile to be used for crunching.'''
        
        self.resulted_in_end = False
        '''Flag marking that the job has resulted in an end of the simulation.'''
  
        
    def is_done(self):
        '''
        Return whether the job is done, i.e. enough crunching has been done.
        '''
        return self.crunching_profile.state_satisfies(self.node.state) or \
               self.resulted_in_end
    
        
    def __repr__(self): #todo: ensure not subclass?
        '''
        Get a string representation of the job.
        
        Example output: 

        Job(node=<garlicsim.data_structures.node.Node with clock 17, untouched,
        belongs to a block, crunched with StepProfile(), at 0x20664b0>,
        crunching_profile=CrunchingProfile(clock_target=100,
        step_profile=StepProfile()))
        '''
        
        stuff = []
        stuff.append("node=%s" % self.node)
        stuff.append("crunching_profile=%s" % self.crunching_profile)
        temp = ", ".join(stuff)
        return ("Job(%s)" % temp)


# Copyright 2009-2010 Ram Rachum.
# This program is distributed under the LGPL2.1 license.

'''
Defines the TreeMember class.

See its documentation for more information.
'''

from garlicsim.general_misc.third_party import abc
    
    
class TreeMember(object):
    '''
    A member of the tree.
    
    This is an abstract base class for all kinds of objects that are members of
    a tree.
    '''
    __metaclass__ = abc.ABCMeta
  
    @abc.abstractmethod
    def __len__(self):
        '''
        Get the length of the tree member.
        
        For a node or an end, this will be 1. For a block, it will be the number
        of contained nodes.
        '''
        raise NotImplementedError

        
    @abc.abstractmethod
    def soft_get_block(self):
        '''
        Get the block that this tree member is on, or itself if it's a block.
        
        If it's not a part of a block, return itself.
        '''
        raise NotImplementedError

    
    @abc.abstractmethod
    def make_containing_path(self):
        '''
        Create a path that contains this tree member.
        
        There may be multiple different paths that contain this tree member.
        This will return the one which points to the newest possible forks.
        Returns the path.
        '''
        raise NotImplementedError
        
    
    @abc.abstractmethod
    def all_possible_paths(self):
        '''
        Get a list of all possible paths that contain this tree member.
        
        Note: There may be paths that contain this tree member which will not be
        identical to one of the paths given here, because these other paths may
        specify decisions that are not even on the same root as these paths.
        '''
        raise NotImplementedError
    
    
    @abc.abstractmethod
    def make_past_path(self):
        '''
        Create a path that contains this tree member.
        
        There may be multiple different paths that contain this tree member.
        This will return a path that doesn't specify any decisions after this
        tree member.
        '''
        raise NotImplementedError


    @abc.abstractmethod
    def get_all_leaves(self, max_nodes_distance=None, max_clock_distance=None):
        '''
        Get all leaves that are descendents of this tree member.
        
        Only leaves with a distance of at most `max_nodes_distance` in nodes or
        `max_clock_distance` in clock are returned. (Note this is an OR
        relation between the two condintions)
        
        Returns a dict of the form:
        
        {
            leaf1: {
                'nodes_distance': nodes_distance1,
                'clock_distance': clock_distance1,
            },            
            leaf2: {
                'nodes_distance': nodes_distance2,
                'clock_distance': clock_distance2,
            },
            # ...
        }
            
        '''
        raise NotImplementedError

    
    @abc.abstractmethod
    def get_ancestor(self, generations=1, round=False):
        '''
        Get an ancestor of this tree member.
        
        `generations` specifies the number of generation that the returned
        ancestor should be above the current tree member. `round` determines how
        this method will behave if it was asked for too many generations back,
        and not enough existed. If `round` is True, it will return the root. If
        `round` is False, it will raise a LookupError.
        '''

        raise NotImplementedError

    
    @abc.abstractmethod
    def get_root(self):
        '''
        Get the root of this tree member.
        
        This means the node which is the parent of the parent of the parent
        of... the parent of this tree member.
        '''
        raise NotImplementedError
    
    
    @abc.abstractmethod
    def is_overlapping(self, tree_member):
        '''
        Return whether this tree member overlaps with the given tree member.
        '''
        raise NotImplementedError
    
        



